import asyncio
import os

import uvloop
from dotenv import load_dotenv
from sqlalchemy import (BigInteger, Column, Integer, MetaData, Table, select)
from sqlalchemy.ext.asyncio import create_async_engine

load_dotenv()

PASSWORD = os.getenv("Postgres_Password")
IP = os.getenv("Postgres_IP")
USER = os.getenv("Postgres_User")
DATABASE = os.getenv("Postgres_Database")
PORT = os.getenv("Postgres_Port")


class DisQuestUsers:
    def __init__(self):
        self.self = self

    async def initTables(self):
        meta = MetaData()
        engineInit = create_async_engine(
            f"postgresql+asyncpg://{USER}:{PASSWORD}@{IP}:{PORT}/{DATABASE}"
        )
        Table(
            "users",
            meta,
            Column("id", BigInteger),
            Column("gid", BigInteger),
            Column("xp", Integer),
        )
        async with engineInit.begin() as connInit:
            await connInit.run_sync(meta.create_all)

    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
    
    async def getxp(self, uid: int, gid: int):
        meta = MetaData()
        engine = create_async_engine(
            f"postgresql+asyncpg://{USER}:{PASSWORD}@{IP}:{PORT}/{DATABASE}"
        )
        users = Table(
            "users",
            meta,
            Column("id", BigInteger),
            Column("gid", BigInteger),
            Column("xp", Integer),
        )
        async with engine.connect() as conn:
            s = select(users.c.xp).where(
                users.c.id == uid, users.c.gid == gid)
            results = await conn.execute(s)
            results_fetched = results.fetchone()
            for row in results_fetched:
                return row
                
    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
                
    async def onInit(self, user_id: int, guild_id: int):
        meta = MetaData()
        engine = create_async_engine(
            f"postgresql+asyncpg://{USER}:{PASSWORD}@{IP}:{PORT}/{DATABASE}"
        )
        users = Table(
            "users",
            meta,
            Column("id", BigInteger),
            Column("gid", BigInteger),
            Column("xp", Integer),
        )
        async with engine.begin() as conn:
            s = select(users.c.xp).filter(users.c.id == user_id).filter(users.c.gid == guild_id)
            results = await conn.execute(s)
            results_fetched = results.fetchone()
            if results_fetched is None:
                insert_new = users.insert().values(xp=0, id=user_id, gid=guild_id)
                await conn.execute(insert_new)
    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())

    async def setxp(self, xp: int, uid: int, gid: int):
        meta = MetaData()
        engine = create_async_engine(
            f"postgresql+asyncpg://{USER}:{PASSWORD}@{IP}:{PORT}/{DATABASE}"
        )
        users = Table(
            "users",
            meta,
            Column("id", BigInteger),
            Column("gid", BigInteger),
            Column("xp", Integer),
        )
        async with engine.begin() as conn:
            update_values = (
                users.update()
                .values(xp=xp)
                .filter(users.c.id == uid)
                .filter(users.c.gid == gid)
            )
            await conn.execute(update_values)
            
    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())

    async def addxp(self, offset, uid: int, gid: int):
        pxp = await self.getxp(uid, gid)
        pxp += offset
        await self.setxp(pxp, uid=uid, gid=gid)
        
    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['disquest_utils']

package_data = \
{'': ['*']}

install_requires = \
['SQLAlchemy>=1.4.39,<2.0.0',
 'asyncpg>=0.26.0,<0.27.0',
 'python-dotenv>=0.20.0,<0.21.0',
 'uvloop>=0.16.0,<0.17.0']

setup_kwargs = {
    'name': 'disquest-utils',
    'version': '1.0.6',
    'description': "A set of async utils for Miku's DisQuest Cog",
    'long_description': '<div align=center>\n\n\n# DisQuest-Utils\n\n![PyPI](https://img.shields.io/pypi/v/disquest-utils?label=PyPi&logo=pypi&logoColor=white) ![PyPI - Python Version](https://img.shields.io/pypi/pyversions/disquest-utils?label=Python&logo=python&logoColor=white)\n![PyPI - License](https://img.shields.io/pypi/l/disquest-utils?label=License&logo=github&logoColor=white)\n\nA set of async utils for DisQuest\n\n<div align=left>\n\n# Info\n\nDisQuest is an base system for storing and giving users on Discord xp. This is just the base set of coroutines needed in order to use DisQuest. DisQuest uses PostgreSQL.\n',
    'author': 'No767',
    'author_email': '73260931+No767@users.noreply.github.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/No767/DisQuest-Utils',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
